require 'spec_helper'
require 'encrypted_yaml/configurator'

describe EncryptedYaml::Configurator do
  before :all do
    Dir.chdir File.join(File.dirname(__FILE__), '..', '..', 'assets')

    @key     = File.read 'two_key'
    @iv      = File.read 'two_iv'
    @encfile = File.read 'two.yaml.enc'
  end

  describe :initialize do
    it 'is a Hash' do
      conf = EncryptedYaml::Configurator.new 'foo.yaml.enc'
      expect(conf).to be_a_kind_of(Hash)
    end

    it 'returns a decrypted version of the encrypted YAML' do
      conf = EncryptedYaml::Configurator.new 'foo.yaml.enc'
      expect(conf[:foo]).to eq([:bar, :baz])
    end

    context 'no options are provided' do
      it 'defaults to loading a keyfile named "key" and an ivfile named "iv"' do
        expect(File).to receive(:read).with('./key').ordered.and_return(@key)
        expect(File).to receive(:read).with('./iv').ordered.and_return(@iv)
        expect(File).to receive(:read).with('foo.yaml.enc').ordered.and_return(@encfile)

        conf = EncryptedYaml::Configurator.new 'foo.yaml.enc'
      end
    end

    context ':keyfile is provided' do
      it 'loads the key from the provided keyfile' do
        expect(File).to receive(:read).with('two_key').ordered.and_return(@key)
        expect(File).to receive(:read).with('./iv').ordered.and_return(@iv)
        expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)

        EncryptedYaml::Configurator.new 'two.yaml.enc', keyfile: 'two_key'
      end

      context ':ivfile is provided' do
        it 'loads the iv from the provided ivfile' do
          expect(File).to receive(:read).with('two_key').ordered.and_return(@key)
          expect(File).to receive(:read).with('two_iv').ordered.and_return(@iv)
          expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)

          EncryptedYaml::Configurator.new 'two.yaml.enc', keyfile: 'two_key', ivfile: 'two_iv'
        end
      end

      context ':iv is provided' do
        it 'uses the provided string as the iv' do
          expect(File).to receive(:read).with('two_key').ordered.and_return(@key)
          expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)
          expect(File).to_not receive(:read).with('two_iv')
          expect(File).to_not receive(:read).with('iv')

          EncryptedYaml::Configurator.new 'two.yaml.enc', keyfile: 'two_key', iv: @iv
        end
      end

      context ':key is provided' do
        it 'prefers the key string and does not attempt to load the keyfile' do
          expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)
          expect(File).to_not receive(:read).with('two_key')
          expect(File).to_not receive(:read).with('key')

          EncryptedYaml::Configurator.new 'two.yaml.enc', keyfile: 'two_key', key: @key, iv: @iv
        end
      end
    end

    context ':key is provided' do
      it 'uses the provided string as the key' do
        expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)
        expect(File).to_not receive(:read).with('two_key')
        expect(File).to_not receive(:read).with('key')

        EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv
      end

      context ':keyfile is provided' do
        it 'prefers :key over :keyfile' do
          expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)
          expect(File).to_not receive(:read).with('two_key')
          expect(File).to_not receive(:read).with('key')

          EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv, keyfile: 'two_key'
        end
      end
    end

    context ':ivfile is provided' do
      it 'loads the ivfile and uses it as the iv' do
        expect(File).to receive(:read).with('two_iv').ordered.and_return(@iv)
        expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)

        EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, ivfile: 'two_iv'
      end

      context ':iv is provided' do
        it 'prefers :iv over :ivfile' do
          expect(File).to receive(:read).with('two.yaml.enc').ordered.and_return(@encfile)
          expect(File).to_not receive(:read).with('two_iv')
          expect(File).to_not receive(:read).with('iv')

          EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, ivfile: 'two_iv', iv: @iv
        end
      end
    end

  end

  describe :reload_config do
    it 'reloads the configuration file' do
      expected_config = {:hello => [:world, :murica]}
      conf = EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv
      expect(conf).to eq(expected_config) # did it load?

      conf.clear
      expect(conf).to eq({}) # is it empty now?

      new_conf = conf.reload_config
      expect(new_conf).to eq(expected_config) # did it load?
    end

    it 'returns a new copy' do
      expected_config = {:hello => [:world, :murica]}
      conf = EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv
      expect(conf).to eq(expected_config) # did it load?

      new_conf = conf.reload_config
      expect(new_conf).to_not be(conf) # is it a new object?
    end
  end

  describe :reload_config! do
    it 'reloads the configuration file' do
      expected_config = {:hello => [:world, :murica]}
      conf = EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv
      expect(conf).to eq(expected_config)

      conf.clear
      expect(conf).to eq({})

      conf.reload_config!
      expect(conf).to eq(expected_config)
    end

    it 'is the same object' do
      conf = EncryptedYaml::Configurator.new 'two.yaml.enc', key: @key, iv: @iv
      new_conf = conf.reload_config!
      expect(new_conf).to be(conf)
    end
  end

end
